@@ -13,15 +13,9 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 """
-from pynetbox.core.response import Record, JsonField
+
+from pynetbox.core.response import JsonField, Record
 
 
 class ConfigContexts(Record):
     data = JsonField
-
-
-class ObjectChanges(Record):
-    object_data = JsonField
-
-    def __str__(self):
-        return self.request_id
